﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Diagnostics;
using System.ServiceProcess;
using System.Text;
using System.Threading;
using System.Management.Instrumentation;

using DojoNorthSoftware.Classic.HomeServerSMART.Components;
using Gurock.SmartInspect;

namespace DojoNorthSoftware.Classic.HomeServerSMART.Service
{
    partial class HSSmartService : ServiceBase
    {
        private bool running = false;
        private HssServiceHelper helper;

        public HSSmartService()
        {
            SiAuto.Main.EnterMethod("DojoNorthSoftware.Classic.HomeServerSMART.Service.HSSmartService");
            SiAuto.Main.LogMessage("Initializing the service: InitializeComponent.");
            InitializeComponent();
            SiAuto.Main.LeaveMethod("DojoNorthSoftware.Classic.HomeServerSMART.Service.HSSmartService");
        }

        /// <summary>
        /// Runs when the service has been signaled by Service Control Manager to start. Start the service in a worker
        /// thread. This way the OnStart method exits quickly. If things take too long to run in the OnStart method, the
        /// SCM may terminate the service.
        /// </summary>
        /// <param name="args"></param>
        protected override void OnStart(string[] args)
        {
            SiAuto.Main.EnterMethod("DojoNorthSoftware.Classic.HomeServerSMART.Service.OnStart");
            SiAuto.Main.LogMessage("Home Server SMART Classic v" + System.Reflection.Assembly.GetExecutingAssembly().GetName().Version.ToString() + " startup in progress");
            SiAuto.Main.LogMessage("Build Name: Sw1tched");
            SiAuto.Main.LogMessage("Copyright © Dojo North Software, LLC 2013");
            Thread t = new Thread(new ThreadStart(ServiceThread));
            t.Name = "HSS_ServiceMain";
            t.Start();
            SiAuto.Main.LogMessage("Home Server SMART/WindowSMART v" + System.Reflection.Assembly.GetExecutingAssembly().GetName().Version.ToString() + " started");
            SiAuto.Main.LeaveMethod("DojoNorthSoftware.Classic.HomeServerSMART.Service.OnStart");
        }

        /// <summary>
        /// Runs when the service has been signaled by Service Control Manager to stop. Set the running flag to false,
        /// which allows the service thread to exit, and thus the service will shut down.
        /// </summary>
        protected override void OnStop()
        {
            SiAuto.Main.EnterMethod("DojoNorthSoftware.Classic.HomeServerSMART.Service.OnStop");
            SiAuto.Main.LogMessage("[Teardown] Home Server SMART has been ordered to halt.");
            running = false;
            SiAuto.Main.LogBool("running", running);
            SiAuto.Main.LogMessage("[Teardown] running flag has been set to false; worker thread will exit.");
            SiAuto.Main.LeaveMethod("DojoNorthSoftware.Classic.HomeServerSMART.Service.OnStop");
        }

        /// <summary>
        /// Runs when the service has been signaled by Service Control Manager to shut down. Set the running flag to false,
        /// which allows the service thread to exit, and thus the service will shut down.
        /// </summary>
        protected override void OnShutdown()
        {
            SiAuto.Main.EnterMethod("DojoNorthSoftware.Classic.HomeServerSMART.Service.OnShutdown");
            SiAuto.Main.LogMessage("[Teardown] Home Server SMART has been ordered to halt.");
            running = false;
            SiAuto.Main.LogBool("running", running);
            SiAuto.Main.LogMessage("[Teardown] running flag has been set to false; worker thread will exit.");
            SiAuto.Main.LeaveMethod("DojoNorthSoftware.Classic.HomeServerSMART.Service.OnShutdown");
        }

        protected override bool OnPowerEvent(PowerBroadcastStatus powerStatus)
        {
            SiAuto.Main.EnterMethod("DojoNorthSoftware.Classic.HomeServerSMART.Service.OnPowerEvent");
            SiAuto.Main.LogWarning("A power state change was detected on the Server.");
            switch (powerStatus)
            {
                case PowerBroadcastStatus.BatteryLow:
                    {
                        SiAuto.Main.LogMessage("Battery power is low.");
                        if (helper != null)
                        {
                            try
                            {
                                helper.PostPowerNotificate("Battery power is low on computer " + Environment.MachineName + ". Consider saving your work or plugging in the computer. If the battery " +
                                    "power gets low enough, the computer may shut down unexpectedly and you could lose any unsaved work.", "Battery Low", true, "Power Event", 65530);
                            }
                            catch
                            {
                            }
                        }
                        break;
                    }
                case PowerBroadcastStatus.OemEvent:
                    {
                        SiAuto.Main.LogMessage("OEM power event detected.");
                        break;
                    }
                case PowerBroadcastStatus.PowerStatusChange:
                    {
                        SiAuto.Main.LogMessage("Power status change - battery to A/C or vice versa.");
                        if (helper != null)
                        {
                            try
                            {
                                helper.PostPowerNotificate("Windows reported a power status change on computer " + Environment.MachineName + ". This can occur if a " +
                                    "computer switches from AC to battery by becoming unplugged or if a power failure occurred while plugged into an uninterruptable " +
                                    "power supply. Windows also reports this condition if the battery percentage changes drastically in a short time. If this is " +
                                    "unexpected, it is possible there is a power outage at this computer's location.", "Power Status Change", true, "Power Event", 65531);
                            }
                            catch
                            {
                            }
                        }
                        break;
                    }
                case PowerBroadcastStatus.QuerySuspend:
                    {
                        SiAuto.Main.LogMessage("System is requesting permission to suspend.");
                        break;
                    }
                case PowerBroadcastStatus.QuerySuspendFailed:
                    {
                        SiAuto.Main.LogMessage("System was denied permission to suspend.");
                        break;
                    }
                case PowerBroadcastStatus.ResumeAutomatic:
                    {
                        SiAuto.Main.LogMessage("System has awakened to handle an automatic event.");
                        break;
                    }
                case PowerBroadcastStatus.ResumeCritical:
                    {
                        SiAuto.Main.LogMessage("System has resumed from a failing battery.");
                        if (helper != null)
                        {
                            try
                            {
                                helper.PostPowerNotificate("Windows reported that computer " + Environment.MachineName + " has resumed from a critical or failing battery. This could be " +
                                    "because you were using the computer and ran the battery too low, causing Windows to force suspension or " +
                                    "hibernation. If this was unexpected, the battery may need to be replaced.", "Battery Critical", true, "Power Event", 65532);
                            }
                            catch
                            {
                            }
                        }
                        break;
                    }
                case PowerBroadcastStatus.ResumeSuspend:
                    {
                        SiAuto.Main.LogMessage("Resume suspend broadcast event; apps can resume full user interaction.");
                        break;
                    }
                case PowerBroadcastStatus.Suspend:
                    {
                        SiAuto.Main.LogMessage("System is about to enter a suspended state.");
                        break;
                    }
                default:
                    {
                        SiAuto.Main.LogMessage("Unknown power event.");
                        break;
                    }
            }
            SiAuto.Main.LeaveMethod("DojoNorthSoftware.Classic.HomeServerSMART.Service.OnPowerEvent");
            return true;
        }

        private void ServiceThread()
        {
            SiAuto.Main.EnterMethod("DojoNorthSoftware.Classic.HomeServerSMART.Service.ServiceThread");
            try
            {
                running = true;
                SiAuto.Main.LogMessage("Instantiate HssServiceHelper in singleton instance.");
                helper = HssServiceHelper.GetInstance();
                SiAuto.Main.LogObject("HssServiceHelper instantiated.", helper);
                int pollingInterval = helper.ServicePollingInterval;
                SiAuto.Main.LogInt("Polling interval is configured.", pollingInterval);
                SiAuto.Main.LogDouble("Default polling interval.", attributeRefresh.Interval);

                if (attributeRefresh.Interval != pollingInterval)
                {
                    SiAuto.Main.LogMessage("Default polling interval and user-defined interval are different; adjusting attributeRefresh interval value.");
                    attributeRefresh.Interval = pollingInterval;
                }
                SiAuto.Main.LogMessage("Start attributeRefresh.");
                attributeRefresh.Start();
                SiAuto.Main.LogMessage("Timer attributeRefresh is started.");

                SiAuto.Main.LogMessage("Starting updateGanderizer. This checks for updates every 2 hours, although the actual web service call only occurs twice per day.");
                updateGanderizer.Start();
                SiAuto.Main.LogMessage("Timer updateGanderizer is started.");

                SiAuto.Main.LogMessage("Checking for Updates.");
                helper.ServiceCheckForUpdates();
                SiAuto.Main.LogMessage("Update check done.");

                SiAuto.Main.LogMessage("Service initialization is complete. Main service thread entering while(running) loop.");
                while (running)
                {
                    Thread.Sleep(1000);
                    pollingInterval = helper.ServicePollingInterval;

                    if (attributeRefresh.Interval != pollingInterval)
                    {
                        SiAuto.Main.LogColored(System.Drawing.Color.Yellow, "User changed polling interval. Worker thread will be restarted.");
                        SiAuto.Main.LogColored(System.Drawing.Color.Yellow, "Stopping attributeRefresh worker process.");
                        attributeRefresh.Stop();
                        SiAuto.Main.LogColored(System.Drawing.Color.Yellow, "Worker process signaled to stop. Sleeping 5 seconds to allow thread to exit.");
                        Thread.Sleep(5000);
                        SiAuto.Main.LogDouble("Old timer interval.", attributeRefresh.Interval);
                        SiAuto.Main.LogInt("New polling interval.", helper.ServicePollingInterval);
                        attributeRefresh.Interval = helper.ServicePollingInterval;
                        SiAuto.Main.LogMessage("Timer interval is reset. Starting attributeRefresh worker process.");
                        Thread.Sleep(1000);
                        attributeRefresh.Start();
                        SiAuto.Main.LogMessage("Workper process is restarted with new polling interval.");
                    }

                    //helper.PerformDriveBenderBitLockerTasks();
                }
                SiAuto.Main.LogMessage("[Teardown] Worker thread detected running flag has been set to false. Beginning demolition.");
                try
                {
                    SiAuto.Main.LogMessage("[Teardown] Signaling the timers to halt.");
                    attributeRefresh.Stop();
                    SiAuto.Main.LogMessage("[Teardown] attributeRefresh has been halted.");
                    updateGanderizer.Stop();
                    SiAuto.Main.LogMessage("[Teardown] updateGanderizer has been halted.");
                }
                catch (Exception timerException)
                {
                    SiAuto.Main.LogError("[Teardown] Exception was thrown signaling the timers to halt: " + timerException);
                    SiAuto.Main.LogException(timerException);
                }
                helper.TearDown();
                SiAuto.Main.LogMessage("[Teardown] The Service Helper has been demolished.");
            }
            catch (Exception ex)
            {
                SiAuto.Main.LogFatal("Exceptions were detected within the main service thread. The service is halting.");
                SiAuto.Main.LogException(ex);
                SiAuto.Main.LogMessage("[Teardown] Attempting demolition of reserved resources to free memory.");
                try
                {
                    SiAuto.Main.LogMessage("[Teardown] Signaling the timers to halt.");
                    attributeRefresh.Stop();
                    SiAuto.Main.LogMessage("[Teardown] attributeRefresh has been halted.");
                    updateGanderizer.Stop();
                    SiAuto.Main.LogMessage("[Teardown] updateGanderizer has been halted.");
                }
                catch (Exception timerException)
                {
                    SiAuto.Main.LogError("[Teardown] Exception was thrown signaling the timers to halt: " + timerException);
                    SiAuto.Main.LogException(timerException);
                }

                try
                {
                    if (helper == null)
                    {
                        SiAuto.Main.LogWarning("[Teardown] Helper object is null; nothing to demolish.");
                    }
                    else
                    {
                        helper.TearDown();
                        SiAuto.Main.LogMessage("[Teardown] The Service Helper has been demolished.");
                    }
                }
                catch (Exception tearDownException)
                {
                    SiAuto.Main.LogError("[Teardown] Demolition failed: " + tearDownException);
                    SiAuto.Main.LogException(tearDownException);
                }
                WindowsEventLogger.LogError("Exceptions were detected in the service worker thread. " + ex.Message +
                    "\n\nThe service is halting.", 53883, Properties.Resources.EventLogJoshua);
            }
            finally
            {
                SiAuto.Main.LeaveMethod("DojoNorthSoftware.Classic.HomeServerSMART.Service.ServiceThread");
            }
        }

        /// <summary>
        /// Event fires when the polling interval elapses. Polling interval is every 5 minutes. Re-reads the configuration
        /// and refreshes all of the data.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void attributeRefresh_Elapsed(object sender, System.Timers.ElapsedEventArgs e)
        {
            SiAuto.Main.EnterMethod("DojoNorthSoftware.Classic.HomeServerSMART.Service.attributeRefresh_Elapsed");
            helper.ServiceAutoPollDisks();
            SiAuto.Main.LeaveMethod("DojoNorthSoftware.Classic.HomeServerSMART.Service.attributeRefresh_Elapsed");
        }

        private void updateGanderizer_Elapsed(object sender, System.Timers.ElapsedEventArgs e)
        {
            SiAuto.Main.EnterMethod("DojoNorthSoftware.Classic.HomeServerSMART.Service.updateGanderizer_Elapsed");
            helper.ServiceCheckForUpdates();
            SiAuto.Main.LeaveMethod("DojoNorthSoftware.Classic.HomeServerSMART.Service.updateGanderizer_Elapsed");
        }
    }
}
